@@ -1,4 +1,4 @@
-#!/usr/bin/env python3.7
+#!/usr/bin/env python3
 import tempfile
 from pyontutils.core import auth
 __doc__ = f"""Use SciGraph to load an ontology from a loacal git repository.
@@ -48,11 +48,13 @@ Options:
     -v --view-defaults              print out the currently configured default values
     -f --graph-config-out=GCO       output for graphload.yaml   [default: {auth.get_path('scigraph-graphload')}]
                                     only useful for `ontload config` ignored otherwise
+    -x --fix-imports-only           prepare graph build but only fix the import chain do not build
 """
 import os
 import json
 import yaml
 import shutil
+import hashlib
 import subprocess
 from io import BytesIO
 from glob import glob
@@ -68,7 +70,7 @@ from docopt import parse_defaults
 from joblib import Parallel, delayed
 from ttlser import CustomTurtleSerializer
 from pyontutils.core import OntGraph
-from pyontutils.utils import noneMembers, TODAY, setPS1, refile, TermColors as tc
+from pyontutils.utils import noneMembers, TODAY, setPS1, refile, TermColors as tc, log
 from pyontutils.namespaces import getCuries, OntCuries
 from pyontutils.hierarchies import creatTree
 from pyontutils.closed_namespaces import rdf, rdfs, owl, skos, oboInOwl, dc
@@ -80,6 +82,7 @@ except NameError:
 defaults = {o.name:o.value if o.argcount else None for o in parse_defaults(__doc__)}
 
 COMMIT_HASH_HEAD_LEN = 8
+CONFIG_HASH_HEAD_LEN = 8
 
 bigleaves = 'go.owl', 'uberon.owl', 'pr.owl', 'doid.owl', 'taxslim.owl', 'chebislim.ttl', 'ero.owl'
 
@@ -89,6 +92,56 @@ class NotBuiltError(FileNotFoundError):
     pass
 
 
+
+def identity_json(blob, *, cls=None,
+                  cypher=hashlib.blake2b, encoding='utf-8', sort_lists=False):
+    """ Return the checksum (default blake2b) of the json string
+    serialize of a python dict, list, etc. This is a hacky substitute
+    for a properly specified algorithm for normalizing and hashing python
+    objects. """
+    # this function may be called without arguments, but if it is
+    # called with arguments then the signature changes so we know that
+    # we are in a different regieme
+    if sort_lists:
+        # reminder that json.JSONEncoder.default is not extensible for
+        # known types >_< SIGH
+        rank = {dict:0,
+                list:1,
+                str:2,
+                float:3,}
+        def key(e):
+            if isinstance(e, dict):
+                return rank[dict], sorted([list(t) for t in e.items()], key=key)
+            elif isinstance(e, list):
+                return rank[list], e  # should already be sorted
+            else:
+                return rank[type(e)], e
+
+        def sigh(blob):
+            if isinstance(blob, dict):
+                return {k: sigh(v) for k, v in blob.items()}
+            elif isinstance(blob, list) or isinstance(blob, tuple):
+                return sorted([sigh(e) for e in blob], key=key)
+            else:
+                return blob
+
+        blob = sigh(blob)
+
+    s = json.dumps(blob,
+                   indent=0,
+                   separators=(',', ':'),
+                   ensure_ascii=True,
+                   sort_keys=True,
+                   cls=cls)
+    snn = s.replace('\n', '')
+
+    b = snn.encode(encoding)
+    m = cypher()
+    m.update(b)
+    checksum = m.digest()
+    return checksum
+
+
 def make_catalog(itrips):
     return '\n'.join(("""<?xml version="1.0" encoding="UTF-8" standalone="no"?>""",
                       """<catalog prefer="public" xmlns="urn:oasis:names:tc:entity:xmlns:xml:catalog">""",
@@ -121,7 +174,10 @@ def getBranch(repo, branch):
 class ReproLoader:
     def __init__(self, zip_location, git_remote, org, git_local, repo_name, branch, commit,
                  remote_base, load_base, graphload_config_template, graphload_ontologies,
-                 patch_config, patch, scigraph_commit, post_clone=lambda: None, check_built=False):
+                 patch_config, patch, scigraph_commit, post_clone=lambda: None,
+                 fix_imports_only=False, check_built=False):
+
+        date_today = TODAY()
 
         load_from_repo=True
         local_base = jpth(git_local, repo_name)
@@ -138,31 +194,34 @@ class ReproLoader:
         else:
             ontology_commit = 'NONE'
 
+        config_path, config = self.make_graphload_config(graphload_config_template, graphload_ontologies, zip_location, date_today)
+        config_hash = identity_json(config, sort_lists=True).hex()
+
         (graph_path, zip_path, zip_command,
          wild_zip_path) = self._set_up_paths(zip_location, repo_name, branch,
-                                             scigraph_commit, ontology_commit)
+                                             scigraph_commit, ontology_commit,
+                                             config_hash, date_today)
 
-        (config, config_path,
-         ontologies) = self.make_graphload_config(graphload_config_template, graphload_ontologies,
-                                                  graph_path, remote_base, local_base, zip_location)
+        # NOTE config is modified in place
+        ontologies = self.configure_config(config, graph_path, remote_base, local_base, config_path)
 
         load_command = load_base.format(config_path=config_path)  # 'exit 1' to test
-        print(load_command)
+        log.info(load_command)
 
         if load_from_repo:
             # replace raw github imports with ontology.neuinfor iris to simplify import chain
             # FIXME this is hardcoded and will not generalize ...
             fix_imports = ("find " + local_base +
                         (" -name '*.ttl' -exec sed -i"
-                            " 's/<http.\+\/ttl\//<http:\/\/ontology.neuinfo.org\/NIF\/ttl\//' {} \;"))
+                            " 's,<http.\+/ttl/,<http://ontology.neuinfo.org/NIF/ttl/,' {} \;"))
             os.system(fix_imports)
 
-        if load_from_repo:
+        if load_from_repo and not fix_imports_only:
             def reset_state(original_branch=nob):
                 repo.git.checkout('--', local_base)
                 original_branch.checkout()
         else:
-            reset_state = lambda x:x
+            reset_state = lambda : None
 
         with execute_regardless(reset_state):  # FIXME start this immediately after we obtain nob?
             # main
@@ -184,7 +243,9 @@ class ReproLoader:
                 pass
 
             maybe_zip_path = glob(wild_zip_path)
-            if not maybe_zip_path:
+            if fix_imports_only:
+                pass
+            elif not maybe_zip_path:
                 if check_built:
                     print('The graph has not been loaded.')
                     raise NotBuiltError('The graph has not been loaded.')
@@ -235,15 +296,16 @@ class ReproLoader:
         return repo, nob
 
     @staticmethod
-    def _set_up_paths(zip_location, repo_name, branch, scigraph_commit, ontology_commit):
+    def _set_up_paths(zip_location, repo_name, branch, scigraph_commit, ontology_commit, config_hash, date_today):
         # TODO consider dumping metadata in a file in the folder too?
         def folder_name(scigraph_commit, wild=False):
             return (repo_name +
                     '-' + branch +
                     '-graph' +
-                    '-' + ('*' if wild else TODAY()) +
+                    '-' + ('*' if wild else date_today) +
                     '-' + scigraph_commit[:COMMIT_HASH_HEAD_LEN] +
-                    '-' + ontology_commit)
+                    '-' + ontology_commit +
+                    '-' + config_hash[:CONFIG_HASH_HEAD_LEN])
 
         def make_folder_zip(wild=False):
             folder = folder_name(scigraph_commit, wild)
@@ -262,9 +324,8 @@ class ReproLoader:
 
     @staticmethod
     def make_graphload_config(graphload_config_template, graphload_ontologies,
-                              graph_path, remote_base, local_base, zip_location,
-                              config_path=None):
-        config_n = 'graphload-' + TODAY() + '.yaml'
+                              zip_location, date_today, config_path=None):
+        config_n = 'graphload-' + date_today + '.yaml'
         config_raw = config_n + '.raw'
         if graphload_ontologies is not None:
             with open(graphload_config_template, 'rt') as f1, open(graphload_ontologies, 'rt') as f2, open(zip_location / config_raw, 'wt') as out:  # LOL PYTHON
@@ -278,30 +339,34 @@ class ReproLoader:
         with open(zip_location / config_raw, 'rt') as f:
             config = yaml.safe_load(f)
 
-        if 'ontologies' not in config:
-            # FIXME log a warning?
-            config['ontologies'] = []
+        if config_path is None:
+            config_path = zip_location / config_n
 
+        return config_path, config
+
+    @staticmethod
+    def configure_config(config, graph_path, remote_base, local_base, config_path):  # lel
         config['graphConfiguration']['location'] = graph_path.as_posix()
         if isinstance(local_base, Path):
             lbasposix = local_base.as_posix()
         else:
             lbasposix = local_base
 
+        if 'ontologies' not in config:
+            # FIXME log a warning?
+            config['ontologies'] = []
+
         config['ontologies'] = [{k:v.replace(remote_base, lbasposix)
                                 if k == 'url'
                                 else v
                                 for k, v in ont.items()}
                                 for ont in config['ontologies']]
 
-        if config_path is None:
-            config_path = zip_location / config_n
-
         with open(config_path, 'wt') as f:
             yaml.dump(config, f, default_flow_style=False)
 
         ontologies = [ont['url'] for ont in config['ontologies']]
-        return config, config_path, ontologies
+        return ontologies
 
 
 def scigraph_build(zip_location, git_remote, org, git_local, branch, commit,
@@ -493,7 +558,7 @@ def local_imports(remote_base, local_base, ontologies, local_versions=tuple(), r
             if ext == '.ttl':
                 infmt = 'turtle'
             else:
-                print(ext, local_filepath)
+                log.info((ext, local_filepath))
                 infmt = None
             if remote:
                 resp = requests.get(local_filepath)  # TODO nonblocking pull these out, fetch, run inner again until done
@@ -504,11 +569,11 @@ def local_imports(remote_base, local_base, ontologies, local_versions=tuple(), r
                         raw = f.read()
                 except FileNotFoundError as e:
                     if local_filepath.startswith('file://'):
-                        print('local_imports has already been run, skipping', local_filepath)
+                        log.info(f'local_imports has already been run, skipping {local_filepath}')
                         return
                         #raise ValueError('local_imports has already been run') from e
                     else:
-                        print(e)  # TODO raise a warning if the file cannot be matched
+                        log.exception(e)  # TODO raise a warning if the file cannot be matched
                         # seems like good practice to have any imported ontology under
                         # version control so all imports are guaranteed to have good
                         # provenance and not split the prior informaiton between the
@@ -517,6 +582,7 @@ def local_imports(remote_base, local_base, ontologies, local_versions=tuple(), r
                         # of the properly tracked files to load as they see fit, but
                         # not add to them (at least in pyontutils land)
                         raw = b''
+
             if oo in raw:  # we only care if there are imports or an ontology iri
                 scratch = OntGraph()
                 if infmt == 'turtle':
@@ -563,7 +629,7 @@ def local_imports(remote_base, local_base, ontologies, local_versions=tuple(), r
                 if not readonly:
                     _orp = CustomTurtleSerializer.roundtrip_prefixes  # FIXME awful hack :/
                     CustomTurtleSerializer.roundtrip_prefixes = True
-                    ttl = scratch.serialize(format='nifttl')
+                    ttl = scratch.serialize(format='nifttl', encoding='utf-8')
                     CustomTurtleSerializer.roundtrip_prefixes = _orp
                     ndata, comment = ttl.split(b'###', 1)
                     out = ndata + b'###' + rest
@@ -571,7 +637,7 @@ def local_imports(remote_base, local_base, ontologies, local_versions=tuple(), r
                         f.write(out)
 
     for start in ontologies:
-        print('START', start)
+        log.info(f'START {start}')
         done.append(start)
         inner(start)
     return sorted(triples)
@@ -725,6 +791,7 @@ def run(args):
     check_built = args['--check-built']
     debug = args['--debug']
     log = args['--logfile']  # TODO
+    fix_imports_only = args['--fix-imports-only']
 
     load_base = 'scigraph-load -c {config_path}'  # now _this_ is easier
 
@@ -761,24 +828,26 @@ def run(args):
                              commit, remote_base, load_base,
                              graphload_config_template, graphload_ontologies,
                              patch_config, patch, scigraph_commit,
-                             check_built=check_built)
+                             fix_imports_only=fix_imports_only,
+                             check_built=check_built,)
 
-        FILE_NAME_ZIP = Path(rl.zip_path).name
-        LATEST = Path(zip_location) / 'LATEST'
-        if LATEST.exists() and LATEST.is_symlink():
-            LATEST.unlink()
+        if not fix_imports_only:
+            FILE_NAME_ZIP = Path(rl.zip_path).name
+            LATEST = Path(zip_location) / 'LATEST'
+            if LATEST.exists() and LATEST.is_symlink():
+                LATEST.unlink()
 
-        LATEST.symlink_to(FILE_NAME_ZIP)
+            LATEST.symlink_to(FILE_NAME_ZIP)
 
-        itrips, config = rl.itrips, rl.config
+            itrips, config = rl.itrips, rl.config
 
-        if not ontologies:
-            ontologies = rl.ontologies
+            if not ontologies:
+                ontologies = rl.ontologies
 
-        print(services_zip)
-        print(rl.zip_path)
-        if '--local' in args:
-            return
+            print(services_zip)
+            print(rl.zip_path)
+            if '--local' in args:
+                return
 
     elif scigraph:
         (scigraph_commit, services_zip,
@@ -790,12 +859,13 @@ def run(args):
             return
 
     elif config:
-        graph_path = Path(args['<graph_path>']).resolve()
+        #graph_path = Path(args['<graph_path>']).resolve()
         config_path = Path(args['--graph-config-out']).resolve()
-        local_base = Path(git_local, repo_name).resolve()
-        ReproLoader.make_graphload_config(graphload_config_template, graphload_ontologies,
-                                          graph_path, remote_base, local_base,
-                                          zip_location, config_path)
+        #local_base = Path(git_local, repo_name).resolve()
+        date_today = TODAY()
+        ReproLoader.make_graphload_config(
+            graphload_config_template, graphload_ontologies,
+            zip_location, date_today, config_path)
 
     elif imports:
         # TODO mismatch between import name and file name needs a better fix
