@@ -49,11 +49,12 @@ def value_nofi(e): return (isinstance(e, ValueError) and
                            e.args and
                            e.args[0].startswith('The file (or absense of file)'))
 
-def value_nova(e): return (isinstance(e, ValueError) and
+def value_nova(e): return (isinstance(e, oa.exceptions.SecretError) and
                            e.args and
-                           e.args[0].startswith('No value found'))
+                           e.args[0].startswith('This secret path'))
 
-def value_val(e): return (isinstance(e, ValueError) and
+def value_val(e): return (isinstance(e, oa.exceptions.SecretError) and
+                          #not breakpoint() and
                           e.args and
                           e.args[0].startswith('Value of secret at'))
 
@@ -89,7 +90,7 @@ errors = {
 def do_test(expect, SCOPES='https://www.googleapis.com/auth/spreadsheets.readonly'):
     try:
         s = sheets._get_oauth_service(SCOPES=SCOPES)
-    except BaseException as e:
+    except Exception as e:
         if not expect(e):
             raise e
 
@@ -120,7 +121,7 @@ class TestGetOauthService(unittest.TestCase):
                 expect = errors[cname, sname]
                 try:
                     do_test(expect)
-                except BaseException as e:
+                except Exception as e:
                     if (cname, sname) == ('user_config_ok', 'secrets_null'):
                         raise e
 
@@ -131,7 +132,7 @@ class TestGetOauthService(unittest.TestCase):
                               if cname in keep_expect or sname in keep_expect else
                               type_scopes)
                     do_test(expect, None)  # FIXME some others come first
-                except BaseException as e:
+                except Exception as e:
                     bads.append((cname, sname, 'SCOPES=None', e))
 
         assert not bads, pprint.pformat(bads)
@@ -152,9 +153,30 @@ class TestSheets(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
+        # ensure we always start from a sane state
+        base_sheet = SheetToTest(readonly=False)
+        start = [['id', 'name', 'other'],
+                 ['a', '', ''],
+                 ['b', '', ''],
+                 ['c', '', ''],
+                 ['d', '', ''],]
+        base_sheet.update(start)
+        base_sheet.commit()
+
+        # new object to avoid any potential lingering state
+        # (we would catch this in the tests below)
         cls.base_sheet = SheetToTest(readonly=False)
+        assert cls.base_sheet.values == start
         cls.base_sheet_ro = SheetToTest()
 
+    @classmethod
+    def tearDownClass(cls):
+        # FIXME shrinks should deal with this themselves
+        # but it is not always obvious whether a call to
+        # update should enqueue a resize requests
+        cls.base_sheet._delete_rest()
+        cls.base_sheet.commit()
+
     def setUp(self):
         self.sheet = SheetToTest(readonly=False, fetch=False)
         self.sheet_ro = SheetToTest(fetch=False)
@@ -195,13 +217,13 @@ class TestSheets(unittest.TestCase):
 
         row.name().value = 'hello there'
         self.sheet.commit()
-        self.sheet_ro.fetch()
+        self.sheet_ro.fetch(fetch_meta=False)
         tv1 = self.sheet_ro.values
         assert self.sheet.values == tv1
 
         row.name().value = ''
         self.sheet.commit()
-        self.sheet_ro.fetch()
+        self.sheet_ro.fetch(fetch_meta=False)
         tv2 = self.sheet_ro.values
         assert self.sheet.values == tv2
         assert tv1 != tv2
@@ -216,13 +238,13 @@ class TestSheets(unittest.TestCase):
         try:
             self.sheet.upsert(row)
             self.sheet.commit()
-            self.sheet_ro.fetch()
+            self.sheet_ro.fetch(fetch_meta=False)
             tv1 = self.sheet_ro.values
             assert self.sheet.values == tv1
         finally:
             self.sheet.upsert(original_values)
             self.sheet.commit()
-            self.sheet_ro.fetch()
+            self.sheet_ro.fetch(fetch_meta=False)
             tv1 = self.sheet_ro.values
             assert self.sheet.values == tv1
 
@@ -234,7 +256,7 @@ class TestSheets(unittest.TestCase):
         try:
             self.sheet.delete(row)
             self.sheet.commit()
-            self.sheet_ro.fetch()
+            self.sheet_ro.fetch(fetch_meta=False)
             tv1 = self.sheet_ro.values
             assert self.sheet.values == tv1
             assert row not in tv1
@@ -242,7 +264,7 @@ class TestSheets(unittest.TestCase):
             if row not in tv1:
                 self.sheet.insert(row)  # FIXME can only append not insert back to previous location ...
                 self.sheet.commit()
-                self.sheet_ro.fetch()
+                self.sheet_ro.fetch(fetch_meta=False)
                 tv1 = self.sheet_ro.values
                 assert self.sheet.values == tv1
                 assert row in tv1
@@ -252,7 +274,7 @@ class TestSheets(unittest.TestCase):
         assert row not in self.sheet.values
         self.sheet.upsert(row)
         self.sheet.commit()
-        self.sheet_ro.fetch()
+        self.sheet_ro.fetch(fetch_meta=False)
         tv1 = self.sheet_ro.values
         try:
             assert self.sheet.values == tv1
@@ -260,7 +282,7 @@ class TestSheets(unittest.TestCase):
             if row in self.sheet.values:
                 self.sheet.delete(row)
                 self.sheet.commit()
-                self.sheet_ro.fetch()
+                self.sheet_ro.fetch(fetch_meta=False)
                 tv1 = self.sheet_ro.values
                 assert self.sheet.values == tv1
 
@@ -276,7 +298,7 @@ class TestSheets(unittest.TestCase):
             self.sheet._appendRow(row)
 
         self.sheet.commit()
-        self.sheet_ro.fetch()
+        self.sheet_ro.fetch(fetch_meta=False)
         tv1 = self.sheet_ro.values
         try:
             assert self.sheet.values == tv1
@@ -289,7 +311,7 @@ class TestSheets(unittest.TestCase):
             if to_delete:
                 self.sheet.delete(*to_delete)
                 self.sheet.commit()
-                self.sheet_ro.fetch()
+                self.sheet_ro.fetch(fetch_meta=False)
                 tv1 = self.sheet_ro.values
                 assert self.sheet.values == tv1
 
@@ -311,24 +333,59 @@ class TestSheets(unittest.TestCase):
 
         oic = self.sheet.index_columns
         self.sheet.index_columns = tuple()
-        self.sheet.values = test_value
+        self.sheet.update(test_value)  # FIXME vs non-truncating self.sheet.values = test_value
+        assert self.sheet._values_less_removed() == test_value
         assert self.sheet.uncommitted()  # FIXME this needs to be run by default every test
         assert ovalues != test_value  # FIXME this needs to be run by default every test
         assert ovalues[0] != test_value[0]
 
         try:
             self.sheet.commit()
-            self.sheet_ro.fetch()
+            self.sheet_ro.fetch(fetch_meta=False)
             tv1 = self.sheet_ro.values
             assert self.sheet.values == tv1
         finally:
             self.sheet.update(ovalues)
             self.sheet.index_columns = oic
             self.sheet.commit()
-            self.sheet_ro.fetch()
+            self.sheet_ro.fetch(fetch_meta=False)
             tv2 = self.sheet_ro.values
             # FIXME I suspect this will break due to missing ends
-            assert self.sheet.values == tv2 == ovalues
+            assert self.sheet.values == tv2
+            assert tv2 == ovalues
+
+    def test_set_values_update_more_cols(self):
+        ovalues = [list(l) for l in self.sheet.values]  # duh mutation is evil
+        _test_value = [[1, 2, 3, 4, 5],  # FIXME these should probably throw errors too
+                       [1, 2, 3, 4, 5],  # since they break the primary key assuptions?
+                       [1, 2, 3, 4, 5],
+                       [1, 2, 3, 4, 5],]
+        # haven't implemented conversion of cell values to strings yet
+        test_value = [[str(c) for c in r] for r in _test_value]
+
+        oic = self.sheet.index_columns
+        self.sheet.index_columns = tuple()
+        self.sheet.update(test_value)  # FIXME vs non-truncating self.sheet.values = test_value
+        assert self.sheet._values_less_removed() == test_value
+        assert self.sheet.uncommitted()  # FIXME this needs to be run by default every test
+        assert ovalues != test_value  # FIXME this needs to be run by default every test
+        assert ovalues[0] != test_value[0]
+
+        try:
+            self.sheet.commit()
+            self.sheet_ro.fetch(fetch_meta=False)
+            tv1 = self.sheet_ro.values
+            assert self.sheet.values == tv1
+        finally:
+            # FIXME need to delete the new rows not just return to the size of the old values
+            self.sheet.update(ovalues)
+            self.sheet.index_columns = oic
+            self.sheet.commit()
+            self.sheet_ro.fetch(fetch_meta=False)
+            tv2 = self.sheet_ro.values
+            # FIXME I suspect this will break due to missing ends
+            assert self.sheet.values == tv2
+            assert tv2 == ovalues
 
     def test_set_values_update_more_rows(self):
         ovalues = [list(l) for l in self.sheet.values]  # duh mutation is evil
@@ -344,14 +401,15 @@ class TestSheets(unittest.TestCase):
 
         oic = self.sheet.index_columns
         self.sheet.index_columns = tuple()
-        self.sheet.values = test_value
+        self.sheet.update(test_value)  # FIXME vs non-truncating self.sheet.values = test_value
+        assert self.sheet._values_less_removed() == test_value
         assert self.sheet.uncommitted()  # FIXME this needs to be run by default every test
         assert ovalues != test_value  # FIXME this needs to be run by default every test
         assert ovalues[0] != test_value[0]
 
         try:
             self.sheet.commit()
-            self.sheet_ro.fetch()
+            self.sheet_ro.fetch(fetch_meta=False)
             tv1 = self.sheet_ro.values
             assert self.sheet.values == tv1
         finally:
@@ -359,10 +417,11 @@ class TestSheets(unittest.TestCase):
             self.sheet.update(ovalues)
             self.sheet.index_columns = oic
             self.sheet.commit()
-            self.sheet_ro.fetch()
+            self.sheet_ro.fetch(fetch_meta=False)
             tv2 = self.sheet_ro.values
             # FIXME I suspect this will break due to missing ends
-            assert self.sheet.values == tv2 == ovalues
+            assert self.sheet.values == tv2
+            assert tv2 == ovalues
 
     def test_row(self):
         r = self.sheet.row_object(0)
